
'''
   We have four players competing in Scrabble+. In each hand
   of Scrabble+, players, in turn, have to enter a word in the
   scoreboard, getting a score that depends on the value
   of the letters composing the inserted word.

   Each player creates his own word by choosing it from a
   hand of 8 tiles, which are replaced once the word
   has been played until they are exhausted. The total number of
   tiles is 130.  The game ends when a player manages to
   finish all the tiles in his hand and there are no more tiles available
   to replace those just played: this happens because
   all the tiles are in the scoreboard and in the hand of
   the other players.

   At the end of the game, the player with the highest score wins,
   considering that for each tile that remains unplayed
   (i.e. it remains in a player's hand when the game ends)
   3 points are subtracted.
   The scores are calculated as follows:
   1 point:  E, A, I, O, N, R, T, L, S, U
   2 points: D, G
   3 points: B, C, M, P
   4 points: F, H, V, W, Y
   5 points: K
   8 points: J, X
  10 points: Q , Z

   Design a function es1(g1, g2, g3, g4, tiles, hand_tiles) that returns the
   scores of a game of Scrabble+ played among the 4 players, with
   the variant that the number of initial letters is "tiles", rather than
   130 and the number of letters available to each player is "hand_tiles".
   g1, g2, g3 and g4 are lists of strings that contain the sequence of
   words inserted by player 1, player 2, player 3 and player 4,
   respectively, in each turn.

ES: hand_tiles=5, tiles=40
    g1 = ['seta','peeks','deter']
    g2 = ['reo','pumas']
    g3 = ['xx','xx']
    g4 = ['frs','bern']

hand_tiles - tiles - word - score
5 5 5 5       20    seta    4 0 0 0
5 5 5 5       16    reo     4 3 0 0
5 5 5 5       13    xx      4 3 16 0
5 5 5 5       11    frs     4 3 16 6
5 5 5 5       8     peeks  15 3 16 6
5 5 5 5       3     pumas  15 12 16 6
5 3 5 5       0     xx     15 12 32 6
5 3 3 5       0     bern   15 12 32 12
5 3 3 1       0     deter  21 12 32 12
0 3 3 1       0     GAME OVER
--------------------------------------
Final score                21  3 23  9
 '''

def ex1(g1, g2, g3, g4, dim_hand, num_letters):
    hands  = [g1, g2, g3, g4]
    tiles  = [dim_hand] * 4
    num_letters -= 4*dim_hand
    punti  = [0] * 4
    i      = 0
    player = i % 4
    mano   = i // 4
    while tiles[player] > 0:
        word = hands[player][mano]
        #assert len(word) <= tiles[player], f"{player=} has to few {tiles[player]=}"
        punti[player] += punteggio(word)
        tiles[player] -= len(word)
        missing = dim_hand - tiles[player]
        if missing <= num_letters:
            num_letters   -= missing
            tiles[player] += missing
        else:
            tiles[player] += num_letters
            num_letters = 0
        if tiles[player] == 0: break
        i += 1
        player = i % 4
        mano   = i // 4
        #print(f"{player=} {mano=} {word=} {tiles[player]=} {punti[player]=} {num_letters=}")
    # togliere i punti per le lettere rimaste in mano
    for j in range(4):
        punti[j] -= 3 * tiles[j]
    return punti

SCORES = {
       1 : 'eaionrtlsu',
       2 : 'dg',
       3 : 'bcmp',
       4 : 'fhvwy',
       5 : 'k',
       8 : 'jx',
      10 : 'qz',
      }
PUNTI = {}
def punteggio(word):
    global PUNTI
    if not PUNTI:
        for v,s in SCORES.items():
            for c in s:
                PUNTI[c] = v
    #return sum( v for c in word for v,s in SCORES.items() if c in s)
    return sum( PUNTI[c] for c in word )

if __name__ == '__main__':
    import json, sys
    with open(sys.argv[1]) as F:
        data = json.load(F)
        data = list(data.values())[0]
    print(data['expected'])
    print(ex1(data['g1'], data['g2'], data['g3'], data['g4'], data['hand_size'], data['num_letters']))

