'''
   Four players play Scrabble+. For each hand
   of Scrabble+, every player enters a word in the
   scoreboard, getting a score that depends on the value
   of the letters composing the inserted word.

   Players create their own words by choosing them from a
   hand of 8 tiles, which are replaced once the word
   has been played until they are exhausted. The total number of
   tiles is 130.  The game ends when a player manages to
   finish all the tiles in their hand and there are no more tiles available
   to replace those just played: this happens because
   all the tiles are in the scoreboard and in the hand of
   the other players.

   At the end of the game, the player with the highest score wins,
   considering that for each tile that remains unplayed
   (that is, which remains in a player's hand when the game ends)
   3 points are subtracted.
   The scores are calculated as follows:
    1 point:  E, A, I, O, N, R, T, L, S, U
    2 points: D, G
    3 points: B, C, M, P
    4 points: F, H, V, W, Y
    5 points: K
    8 points: J, X
   10 points: Q , Z

   Design a function ex1(g1, g2, g3, g4, dim_hand, num_letters) that returns the
   scores of a game of Scrabble+ played among the four players, with
   the variant that the number of initial letters is "num_letters" instead of
   130, and that the number of letters available to each player is "dim_hand".
   g1, g2, g3 and g4 are lists of strings that contain the sequence of
   words inserted by player 1, player 2, player 3 and player 4,
   respectively, in every round.

Ex.: dim_hand=5, num_letters=40
     g1 = ['seta','peeks','deter']
     g2 = ['reo','pumas']
     g3 = ['xx','xx']
     g4 = ['frs','bern']
     
     Notice that 5 tiles are given to each of the four players already
     at the beginning of the game, thus num_letters decreases accordingly.

dim_hand - num_letters - word - score
5 5 5 5    20            seta    4  0  0  0
5 5 5 5    16            reo     4  3  0  0
5 5 5 5    13            xx      4  3 16  0
5 5 5 5    11            frs     4  3 16  6
5 5 5 5     8            peeks  15  3 16  6
5 5 5 5     3            pumas  15 12 16  6
5 3 5 5     0            xx     15 12 32  6
5 3 3 5     0            bern   15 12 32 12
5 3 3 1     0            deter  21 12 32 12
0 3 3 1     0                     GAME OVER
-------------------------------------------
Final score                     21  3 23  9

The TIMEOUT for each test is 0.5 seconds.

NOTICE: it’s forbidden to:
    - import other libraries
    - use global variables
    - open files
'''
def ex1(g1, g2, g3, g4, dim_hand, num_letters):
    scorecard = compute_scorecard()
    tiles_in_hands = [dim_hand] * 4 # g1 → tiles_in_hands[0], … g4 → tiles_in_hands[3]
    scores = [0] * 4 # g1 → scores[0], … g4 → scores[3]
    games = [g1, g2, g3, g4] # A list of lists with all players
    round_counter = 0 # The counter of the game
    player = 0 # Player number
    final_round = False
    
    # Initialise the game: tiles are distributed among players
    num_letters -= dim_hand * 4
    
    while not(final_round):
        (final_round, tiles_in_hands, num_letters, scores) =\
            play_round(games[player][round_counter], player,
                       tiles_in_hands, dim_hand, num_letters, scores, scorecard)
        player += 1
        if player >= len(scores):
            player = 0
            round_counter += 1

    # At the end of the game, apply penalties
    for i in range(len(scores)):
        scores[i] -= compute_penalty(tiles_in_hands[i])

    return scores


def play_round(played_word, player,
               tiles_in_hands, dim_hand, num_letters, scores, scorecard):
    # We assume we are not in the last round at the beginning
    final_round = False

    # Update the player’s score
    scores[player] += compute_score(played_word, scorecard)
    # Remove used tiles from player’s hands
    tiles_in_hands[player] -= len(played_word)
    # Give the player as many tiles as possible to fill the hand again
    replacement_tiles = min(dim_hand - tiles_in_hands[player], num_letters)
    tiles_in_hands[player] += replacement_tiles
    # Update the count of available tiles
    num_letters -= replacement_tiles

    # If a hand is empty and there are no more tiles available, game over!
    final_round = tiles_in_hands[player] == 0 and num_letters == 0

    return (final_round, tiles_in_hands, num_letters, scores)


def compute_penalty(tiles_in_hand):
    return 3 * tiles_in_hand


def compute_score(word, scorecard):
    score = 0
    for char in word:
        score += scorecard[ord(char)]

    return score

def compute_scorecard():
    scorecard = [0] * 127
    #  1 point:  E, A, I, O, N, R, T, L, S, U
    for char in "EAIONRTLSU":
        scorecard[ord(char)] = scorecard[ord(char.lower())] = 1
    #  2 points: D, G
    for char in "DG" + "dg":
        scorecard[ord(char)] =  2
    #  3 points: B, C, M, P
    for char in "BCMP" + "bcmp":
        scorecard[ord(char)] =  3
    #  4 points; F, H, V, W, Y
    for char in "FHVWY":
        scorecard[ord(char)] = scorecard[ord(char.lower())] = 4
    #  5 points: K
    for char in "Kk":
        scorecard[ord(char)] =  5
    #  8 points: J, X
    for char in "JjXx":
        scorecard[ord(char)] =  8
    # 10 points: Q , Z
    for char in "QZqz":
        scorecard[ord(char)] = 10
    
    return scorecard
      

if __name__ == "__main__":
    # Place your own tests here
    pass
